@@ -1,6 +1,6 @@
 """
 Cython wrapper to provide python interfaces to
-PROJ.4 (https://github.com/OSGeo/proj.4/wiki) functions.
+PROJ (https://proj.org) functions.
 
 Performs geodetic computations.
 
@@ -35,92 +35,11 @@ import math
 
 from pyproj._geod import Geod as _Geod
 from pyproj._geod import geodesic_version_str
+from pyproj._list import get_ellps_map
+from pyproj.exceptions import GeodError
 from pyproj.utils import _convertback, _copytobuffer
 
-pj_ellps = {
-    "MERIT": {"a": 6378137.0, "rf": 298.257, "description": "MERIT 1983"},
-    "SGS85": {
-        "a": 6378136.0,
-        "rf": 298.257,
-        "description": "Soviet Geodetic System 85",
-    },
-    "GRS80": {
-        "a": 6378137.0,
-        "rf": 298.257222101,
-        "description": "GRS 1980(IUGG, 1980)",
-    },
-    "IAU76": {"a": 6378140.0, "rf": 298.257, "description": "IAU 1976"},
-    "airy": {"a": 6377563.396, "b": 6356256.910, "description": "Airy 1830"},
-    "APL4.9": {"a": 6378137.0, "rf": 298.25, "description": "Appl. Physics. 1965"},
-    "NWL9D": {"a": 6378145.0, "rf": 298.25, "description": " Naval Weapons Lab., 1965"},
-    "mod_airy": {"a": 6377340.189, "b": 6356034.446, "description": "Modified Airy"},
-    "andrae": {
-        "a": 6377104.43,
-        "rf": 300.0,
-        "description": "Andrae 1876 (Den., Iclnd.)",
-    },
-    "aust_SA": {
-        "a": 6378160.0,
-        "rf": 298.25,
-        "description": "Australian Natl & S. Amer. 1969",
-    },
-    "GRS67": {"a": 6378160.0, "rf": 298.2471674270, "description": "GRS 67(IUGG 1967)"},
-    "bessel": {"a": 6377397.155, "rf": 299.1528128, "description": "Bessel 1841"},
-    "bess_nam": {
-        "a": 6377483.865,
-        "rf": 299.1528128,
-        "description": "Bessel 1841 (Namibia)",
-    },
-    "clrk66": {"a": 6378206.4, "b": 6356583.8, "description": "Clarke 1866"},
-    "clrk80": {"a": 6378249.145, "rf": 293.4663, "description": "Clarke 1880 mod."},
-    "CPM": {
-        "a": 6375738.7,
-        "rf": 334.29,
-        "description": "Comm. des Poids et Mesures 1799",
-    },
-    "delmbr": {"a": 6376428.0, "rf": 311.5, "description": "Delambre 1810 (Belgium)"},
-    "engelis": {"a": 6378136.05, "rf": 298.2566, "description": "Engelis 1985"},
-    "evrst30": {"a": 6377276.345, "rf": 300.8017, "description": "Everest 1830"},
-    "evrst48": {"a": 6377304.063, "rf": 300.8017, "description": "Everest 1948"},
-    "evrst56": {"a": 6377301.243, "rf": 300.8017, "description": "Everest 1956"},
-    "evrst69": {"a": 6377295.664, "rf": 300.8017, "description": "Everest 1969"},
-    "evrstSS": {
-        "a": 6377298.556,
-        "rf": 300.8017,
-        "description": "Everest (Sabah & Sarawak)",
-    },
-    "fschr60": {
-        "a": 6378166.0,
-        "rf": 298.3,
-        "description": "Fischer (Mercury Datum) 1960",
-    },
-    "fschr60m": {"a": 6378155.0, "rf": 298.3, "description": "Modified Fischer 1960"},
-    "fschr68": {"a": 6378150.0, "rf": 298.3, "description": "Fischer 1968"},
-    "helmert": {"a": 6378200.0, "rf": 298.3, "description": "Helmert 1906"},
-    "hough": {"a": 6378270.0, "rf": 297.0, "description": "Hough"},
-    "intl": {
-        "a": 6378388.0,
-        "rf": 297.0,
-        "description": "International 1909 (Hayford)",
-    },
-    "krass": {"a": 6378245.0, "rf": 298.3, "description": "Krassovsky, 1942"},
-    "kaula": {"a": 6378163.0, "rf": 298.24, "description": "Kaula 1961"},
-    "lerch": {"a": 6378139.0, "rf": 298.257, "description": "Lerch 1979"},
-    "mprts": {"a": 6397300.0, "rf": 191.0, "description": "Maupertius 1738"},
-    "new_intl": {
-        "a": 6378157.5,
-        "b": 6356772.2,
-        "description": "New International 1967",
-    },
-    "plessis": {"a": 6376523.0, "b": 6355863.0, "description": "Plessis 1817 (France)"},
-    "SEasia": {"a": 6378155.0, "b": 6356773.3205, "description": "Southeast Asia"},
-    "walbeck": {"a": 6376896.0, "b": 6355834.8467, "description": "Walbeck"},
-    "WGS60": {"a": 6378165.0, "rf": 298.3, "description": "WGS 60"},
-    "WGS66": {"a": 6378145.0, "rf": 298.25, "description": "WGS 66"},
-    "WGS72": {"a": 6378135.0, "rf": 298.26, "description": "WGS 72"},
-    "WGS84": {"a": 6378137.0, "rf": 298.257223563, "description": "WGS 84"},
-    "sphere": {"a": 6370997.0, "b": 6370997.0, "description": "Normal Sphere"},
-}
+pj_ellps = get_ellps_map()
 
 
 class Geod(_Geod):
@@ -135,6 +54,22 @@ class Geod(_Geod):
     the 'inv' method) involves determining the forward and back
     azimuths and distance given the latitudes and longitudes of an
     initial and terminus point.
+
+    Attributes
+    ----------
+    initstring: str
+        The string form of the user input used to create the Geod.
+    sphere: bool
+        If True, it is a sphere.
+    a: float
+        The ellipsoid equatorial radius, or semi-major axis.
+    b: float
+        The ellipsoid polar radius, or semi-minor axis.
+    es: float
+        The 'eccentricity' of the ellipse, squared (1-b2/a2).
+    f: float
+        The ellipsoid 'flattening' parameter ( (a-b)/a ).
+
     """
 
     def __init__(self, initstring=None, **kwargs):
@@ -143,56 +78,10 @@ class Geod(_Geod):
 
         Geodetic parameters for specifying the ellipsoid
         can be given in a dictionary 'initparams', as keyword arguments,
-        or as as proj4 geod initialization string.
-        Following is a list of the ellipsoids that may be defined using the
-        'ellps' keyword (these are stored in the model variable pj_ellps)::
+        or as as proj geod initialization string.
 
-           MERIT a=6378137.0      rf=298.257       MERIT 1983
-           SGS85 a=6378136.0      rf=298.257       Soviet Geodetic System 85
-           GRS80 a=6378137.0      rf=298.257222101 GRS 1980(IUGG, 1980)
-           IAU76 a=6378140.0      rf=298.257       IAU 1976
-           airy a=6377563.396     b=6356256.910    Airy 1830
-           APL4.9 a=6378137.0.    rf=298.25        Appl. Physics. 1965
-           airy a=6377563.396     b=6356256.910    Airy 1830
-           APL4.9 a=6378137.0.    rf=298.25        Appl. Physics. 1965
-           NWL9D a=6378145.0.     rf=298.25        Naval Weapons Lab., 1965
-           mod_airy a=6377340.189 b=6356034.446    Modified Airy
-           andrae a=6377104.43    rf=300.0         Andrae 1876 (Den., Iclnd.)
-           aust_SA a=6378160.0    rf=298.25        Australian Natl & S. Amer. 1969
-           GRS67 a=6378160.0      rf=298.247167427 GRS 67(IUGG 1967)
-           bessel a=6377397.155   rf=299.1528128   Bessel 1841
-           bess_nam a=6377483.865 rf=299.1528128   Bessel 1841 (Namibia)
-           clrk66 a=6378206.4     b=6356583.8      Clarke 1866
-           clrk80 a=6378249.145   rf=293.4663      Clarke 1880 mod.
-           CPM a=6375738.7        rf=334.29        Comm. des Poids et Mesures 1799
-           delmbr a=6376428.      rf=311.5         Delambre 1810 (Belgium)
-           engelis a=6378136.05   rf=298.2566      Engelis 1985
-           evrst30 a=6377276.345  rf=300.8017      Everest 1830
-           evrst48 a=6377304.063  rf=300.8017      Everest 1948
-           evrst56 a=6377301.243  rf=300.8017      Everest 1956
-           evrst69 a=6377295.664  rf=300.8017      Everest 1969
-           evrstSS a=6377298.556  rf=300.8017      Everest (Sabah & Sarawak)
-           fschr60 a=6378166.     rf=298.3         Fischer (Mercury Datum) 1960
-           fschr60m a=6378155.    rf=298.3         Modified Fischer 1960
-           fschr68 a=6378150.     rf=298.3         Fischer 1968
-           helmert a=6378200.     rf=298.3         Helmert 1906
-           hough a=6378270.0      rf=297.          Hough
-           helmert a=6378200.     rf=298.3         Helmert 1906
-           hough a=6378270.0      rf=297.          Hough
-           intl a=6378388.0       rf=297.          International 1909 (Hayford)
-           krass a=6378245.0      rf=298.3         Krassovsky, 1942
-           kaula a=6378163.       rf=298.24        Kaula 1961
-           lerch a=6378139.       rf=298.257       Lerch 1979
-           mprts a=6397300.       rf=191.          Maupertius 1738
-           new_intl a=6378157.5   b=6356772.2      New International 1967
-           plessis a=6376523.     b=6355863.       Plessis 1817 (France)
-           SEasia a=6378155.0     b=6356773.3205   Southeast Asia
-           walbeck a=6376896.0    b=6355834.8467   Walbeck
-           WGS60 a=6378165.0      rf=298.3         WGS 60
-           WGS66 a=6378145.0      rf=298.25        WGS 66
-           WGS72 a=6378135.0      rf=298.26        WGS 72
-           WGS84 a=6378137.0      rf=298.257223563 WGS 84
-           sphere a=6370997.0     b=6370997.0      Normal Sphere (r=6370997)
+        You can get a dictionary of ellipsoids using :func:`~pyproj.get_ellps_map`
+        or with the variable `pyproj.pj_ellps`.
 
         The parameters of the ellipsoid may also be set directly using
         the 'a' (semi-major or equatorial axis radius) keyword, and
@@ -200,10 +89,8 @@ class Geod(_Geod):
         or polar axis radius), 'e' (eccentricity), 'es' (eccentricity
         squared), 'f' (flattening), or 'rf' (reciprocal flattening).
 
-        See the proj documentation (https://github.com/OSGeo/proj.4/wiki) for more
-        information about specifying ellipsoid parameters (specifically,
-        the chapter 'Specifying the Earth's figure' in the main Proj
-        users manual).
+        See the proj documentation (https://proj.org) for more
+        information about specifying ellipsoid parameters.
 
         Example usage:
 
@@ -230,7 +117,8 @@ class Geod(_Geod):
         >>> lons2 = [boston_lon, portland_lon, london_lon]
         >>> lats2 = [boston_lat, portland_lat, london_lat]
         >>> az12,az21,dist = g.inv(lons1,lats1,lons2,lats2)
-        >>> for faz,baz,d in list(zip(az12,az21,dist)): "%7.3f %7.3f %9.3f" % (faz,baz,d)
+        >>> for faz, baz, d in list(zip(az12,az21,dist)):
+        ...     "%7.3f %7.3f %9.3f" % (faz, baz, d)
         ' 54.663 -123.448 288303.720'
         '-65.463  79.342 4013037.318'
         ' 51.254 -71.576 5579916.651'
@@ -329,7 +217,7 @@ class Geod(_Geod):
         iny, yisfloat, yislist, yistuple = _copytobuffer(lats)
         inz, zisfloat, zislist, zistuple = _copytobuffer(az)
         ind, disfloat, dislist, distuple = _copytobuffer(dist)
-        super(Geod, self)._fwd(inx, iny, inz, ind, radians=radians)
+        self._fwd(inx, iny, inz, ind, radians=radians)
         # if inputs were lists, tuples or floats, convert back.
         outx = _convertback(xisfloat, xislist, xistuple, inx)
         outy = _convertback(yisfloat, yislist, xistuple, iny)
@@ -353,7 +241,7 @@ class Geod(_Geod):
         iny, yisfloat, yislist, yistuple = _copytobuffer(lats1)
         inz, zisfloat, zislist, zistuple = _copytobuffer(lons2)
         ind, disfloat, dislist, distuple = _copytobuffer(lats2)
-        super(Geod, self)._inv(inx, iny, inz, ind, radians=radians)
+        self._inv(inx, iny, inz, ind, radians=radians)
         # if inputs were lists, tuples or floats, convert back.
         outx = _convertback(xisfloat, xislist, xistuple, inx)
         outy = _convertback(yisfloat, yislist, xistuple, iny)
@@ -394,7 +282,14 @@ class Geod(_Geod):
         >>> import math
         >>> dg2rad = math.radians(1.)
         >>> rad2dg = math.degrees(1.)
-        >>> lonlats = g.npts(dg2rad*boston_lon,dg2rad*boston_lat,dg2rad*portland_lon,dg2rad*portland_lat,10,radians=True)
+        >>> lonlats = g.npts(
+        ...    dg2rad*boston_lon,
+        ...    dg2rad*boston_lat,
+        ...    dg2rad*portland_lon,
+        ...    dg2rad*portland_lat,
+        ...    10,
+        ...    radians=True
+        ... )
         >>> for lon,lat in lonlats: '%6.3f  %7.3f' % (rad2dg*lat, rad2dg*lon)
         '43.528  -75.414'
         '44.637  -79.883'
@@ -412,6 +307,233 @@ class Geod(_Geod):
         )
         return list(zip(lons, lats))
 
+    def line_length(self, lons, lats, radians=False):
+        """
+        Calculate the total distance between points along a line.
+
+        >>> from pyproj import Geod
+        >>> geod = Geod('+a=6378137 +f=0.0033528106647475126')
+        >>> lats = [-72.9, -71.9, -74.9, -74.3, -77.5, -77.4, -71.7, -65.9, -65.7,
+        ...         -66.6, -66.9, -69.8, -70.0, -71.0, -77.3, -77.9, -74.7]
+        >>> lons = [-74, -102, -102, -131, -163, 163, 172, 140, 113,
+        ...         88, 59, 25, -4, -14, -33, -46, -61]
+        >>> total_length = geod.line_length(lons, lats)
+        >>> "{:.3f}".format(total_length)
+        '14259605.611'
+
+
+        Parameters
+        ----------
+        lons: array, :class:`numpy.ndarray`, list, tuple, or scalar
+            The longitude points along a line.
+        lats: array, :class:`numpy.ndarray`, list, tuple, or scalar
+            The latitude points along a line.
+        radians: bool, optional
+            If True, the input data is assumed to be in radians.
+
+        Returns
+        -------
+        float: The total length of the line.
+        """
+        # process inputs, making copies that support buffer API.
+        inx, xisfloat, xislist, xistuple = _copytobuffer(lons)
+        iny, yisfloat, yislist, yistuple = _copytobuffer(lats)
+        return self._line_length(inx, iny, radians=radians)
+
+    def line_lengths(self, lons, lats, radians=False):
+        """
+        Calculate the distances between points along a line.
+
+        >>> from pyproj import Geod
+        >>> geod = Geod(ellps="WGS84")
+        >>> lats = [-72.9, -71.9, -74.9]
+        >>> lons = [-74, -102, -102]
+        >>> for line_length in geod.line_lengths(lons, lats):
+        ...     "{:.3f}".format(line_length)
+        '943065.744'
+        '334805.010'
+
+        Parameters
+        ----------
+        lons: array, :class:`numpy.ndarray`, list, tuple, or scalar
+            The longitude points along a line.
+        lats: array, :class:`numpy.ndarray`, list, tuple, or scalar
+            The latitude points along a line.
+        radians: bool, optional
+            If True, the input data is assumed to be in radians.
+
+        Returns
+        -------
+        array, :class:`numpy.ndarray`, list, tuple, or scalar:
+            The total length of the line.
+        """
+        # process inputs, making copies that support buffer API.
+        inx, xisfloat, xislist, xistuple = _copytobuffer(lons)
+        iny, yisfloat, yislist, yistuple = _copytobuffer(lats)
+        self._line_length(inx, iny, radians=radians)
+        line_lengths = _convertback(xisfloat, xislist, xistuple, inx)
+        return line_lengths if xisfloat else line_lengths[:-1]
+
+    def polygon_area_perimeter(self, lons, lats, radians=False):
+        """
+        A simple interface for computing the area (meters^2) and perimeter (meters)
+        of a geodesic polygon.
+
+        .. warning:: Only simple polygons (which are not self-intersecting) are allowed.
+
+        .. note:: lats should be in the range [-90 deg, 90 deg].
+
+        There's no need to "close" the polygon by repeating the first vertex.
+        The area returned is signed with counter-clockwise traversal being treated as
+        positive.
+
+        Example usage:
+
+        >>> from pyproj import Geod
+        >>> geod = Geod('+a=6378137 +f=0.0033528106647475126')
+        >>> lats = [-72.9, -71.9, -74.9, -74.3, -77.5, -77.4, -71.7, -65.9, -65.7,
+        ...         -66.6, -66.9, -69.8, -70.0, -71.0, -77.3, -77.9, -74.7]
+        >>> lons = [-74, -102, -102, -131, -163, 163, 172, 140, 113,
+        ...         88, 59, 25, -4, -14, -33, -46, -61]
+        >>> poly_area, poly_perimeter = geod.polygon_area_perimeter(lons, lats)
+        >>> "{:.1f} {:.1f}".format(poly_area, poly_perimeter)
+        '13376856682207.4 14710425.4'
+
+
+        Parameters
+        ----------
+        lons: array, :class:`numpy.ndarray`, list, tuple, or scalar
+            An array of longitude values.
+        lats: array, :class:`numpy.ndarray`, list, tuple, or scalar
+            An array of latitude values.
+        radians: bool, optional
+            If True, the input data is assumed to be in radians.
+
+        Returns
+        -------
+        (float, float):
+            The geodesic area (meters^2) and permimeter (meters) of the polygon.
+        """
+        return self._polygon_area_perimeter(
+            _copytobuffer(lons)[0], _copytobuffer(lats)[0], radians=radians
+        )
+
+    def geometry_length(self, geometry, radians=False):
+        """
+        Returns the geodesic length (meters) of the shapely geometry.
+
+        If it is a Polygon, it will return the sum of the
+        lengths along the perimeter.
+        If it is a MultiPolygon or MultiLine, it will return
+        the sum of the lengths.
+
+        Example usage:
+
+        >>> from pyproj import Geod
+        >>> from shapely.geometry import Point, LineString
+        >>> line_string = LineString([Point(1, 2), Point(3, 4)])
+        >>> geod = Geod(ellps="WGS84")
+        >>> "{:.3f}".format(geod.geometry_length(line_string))
+        '313588.397'
+
+        Parameters
+        ----------
+        geometry: :class:`shapely.geometry.BaseGeometry`
+            The geometry to calculate the length from.
+        radians: bool, optional
+            If True, the input data is assumed to be in radians.
+
+        Returns
+        -------
+        float: The total geodesic length of the geometry (meters).
+
+        """
+        try:
+            return self.line_length(*geometry.xy, radians=radians)
+        except (AttributeError, NotImplementedError):
+            pass
+        if hasattr(geometry, "exterior"):
+            return self.geometry_length(geometry.exterior, radians=radians)
+        elif hasattr(geometry, "geoms"):
+            total_length = 0.0
+            for geom in geometry.geoms:
+                total_length += self.geometry_length(geom, radians=radians)
+            return total_length
+        raise GeodError("Invalid geometry provided.")
+
+    def geometry_area_perimeter(self, geometry, radians=False):
+        """
+        A simple interface for computing the area (meters^2) and perimeter (meters)
+        of a geodesic polygon as a shapely geometry.
+
+        .. warning:: Only simple polygons (which are not self-intersecting) are allowed.
+
+        .. note:: lats should be in the range [-90 deg, 90 deg].
+
+        There's no need to "close" the polygon by repeating the first vertex.
+        The area returned is signed with counter-clockwise traversal being treated as
+        positive.
+
+        If it is a Polygon, it will return the area and exterior perimeter.
+        It will subtract the area of the interior holes.
+        If it is a MultiPolygon or MultiLine, it will return
+        the sum of the areas and perimeters of all geometries.
+
+
+        Example usage:
+
+        >>> from pyproj import Geod
+        >>> from shapely.geometry import LineString, Point, Polygon
+        >>> geod = Geod(ellps="WGS84")
+        >>> poly_area, poly_perimeter = geod.geometry_area_perimeter(
+        ...     Polygon(
+        ...         LineString([
+        ...             Point(1, 1), Point(1, 10), Point(10, 10), Point(10, 1)
+        ...         ]),
+        ...         holes=[LineString([Point(1, 2), Point(3, 4), Point(5, 2)])],
+        ...     )
+        ... )
+        >>> "{:.3f} {:.3f}".format(poly_area, poly_perimeter)
+        '-944373881400.339 3979008.036'
+
+
+        Parameters
+        ----------
+        geometry: :class:`shapely.geometry.BaseGeometry`
+            The geometry to calculate the area and perimeter from.
+        radians: bool, optional
+            If True, the input data is assumed to be in radians.
+
+         Returns
+        -------
+        (float, float):
+            The geodesic area (meters^2) and permimeter (meters) of the polygon.
+       """
+        try:
+            return self.polygon_area_perimeter(*geometry.xy, radians=radians)
+        except (AttributeError, NotImplementedError):
+            pass
+        # polygon
+        if hasattr(geometry, "exterior"):
+            total_area, total_perimeter = self.geometry_area_perimeter(
+                geometry.exterior, radians=radians
+            )
+            # subtract area of holes
+            for hole in geometry.interiors:
+                area, _ = self.geometry_area_perimeter(hole, radians=radians)
+                total_area -= area
+            return total_area, total_perimeter
+        # multi geometries
+        elif hasattr(geometry, "geoms"):
+            total_area = 0.0
+            total_perimeter = 0.0
+            for geom in geometry.geoms:
+                area, perimeter = self.geometry_area_perimeter(geom, radians=radians)
+                total_area += area
+                total_perimeter += perimeter
+            return total_area, total_perimeter
+        raise GeodError("Invalid geometry provided.")
+
     def __repr__(self):
         # search for ellipse name
         for (ellps, vals) in pj_ellps.items():
@@ -435,12 +557,16 @@ class Geod(_Geod):
         Example usage:
 
         >>> from pyproj import Geod
-        >>> gclrk1 = Geod(ellps='clrk66') # Use Clarke 1866 ellipsoid.
-        >>> gclrk2 = Geod(a=6378206.4, b=6356583.8) # Define Clarke 1866 using parameters
+        >>> # Use Clarke 1866 ellipsoid.
+        >>> gclrk1 = Geod(ellps='clrk66')
+        >>> # Define Clarke 1866 using parameters
+        >>> gclrk2 = Geod(a=6378206.4, b=6356583.8)
         >>> gclrk1 == gclrk2
         True
-        >>> gwgs66 = Geod('+ellps=WGS66')  # WGS 66 ellipsoid, Proj.4 style
-        >>> gnwl9d = Geod('+ellps=NWL9D')  # Naval Weapons Lab., 1965 ellipsoid
+        >>> # WGS 66 ellipsoid, PROJ style
+        >>> gwgs66 = Geod('+ellps=WGS66')
+        >>> # Naval Weapons Lab., 1965 ellipsoid
+        >>> gnwl9d = Geod('+ellps=NWL9D')
         >>> # these ellipsoids are the same
         >>> gnwl9d == gwgs66
         True
