@@ -369,12 +369,6 @@ class Geod_NaN_Issue112_Test(unittest.TestCase):
         self.assertTrue(azi2 == azi2)
 
 
-def test_proj_version():
-    awips221 = Proj(proj="lcc", R=6371200, lat_1=50, lat_2=50, lon_0=-107)
-    with pytest.warns(DeprecationWarning, match="'Proj.proj_version' is deprecated"):
-        assert type(awips221.proj_version) is int
-
-
 def test_proj_equals():
     assert Proj(4326) == Proj("epsg:4326")
     assert Proj(4326) != Proj("epsg:3857")
