@@ -6,6 +6,7 @@ import pyproj
 from pyproj import Proj, Transformer, itransform, transform
 from pyproj.enums import TransformDirection
 from pyproj.exceptions import ProjError
+from pyproj.transformer import AreaOfInterest, TransformerGroup
 
 
 def test_tranform_wgs84_to_custom():
@@ -415,13 +416,99 @@ def test_str():
 
 def test_repr():
     assert repr(Transformer.from_crs(7789, 8401)) == (
-        "<Transformation Transformer: helmert>\n" "ITRF2014 to ETRF2014 (1)"
+        "<Transformation Transformer: helmert>\n"
+        "Description: ITRF2014 to ETRF2014 (1)\n"
+        "Area of Use:\n"
+        "- name: Europe - ETRS89\n"
+        "- bounds: (-16.1, 32.88, 40.18, 84.17)"
     )
 
     assert repr(Transformer.from_crs(4326, 3857)) == (
-        "<Conversion Transformer: pipeline>\n" "Popular Visualisation Pseudo-Mercator"
+        "<Conversion Transformer: pipeline>\n"
+        "Description: Popular Visualisation Pseudo-Mercator\n"
+        "Area of Use:\n"
+        "- name: World\n"
+        "- bounds: (-180.0, -90.0, 180.0, 90.0)"
     )
 
     assert repr(Transformer.from_crs(4326, 26917)) == (
-        "<Unknown Transformer: unknown>\n" "unavailable until proj_trans is called"
+        "<Unknown Transformer: unknown>\n"
+        "Description: unavailable until proj_trans is called\n"
+        "Area of Use:\n- undefined"
     )
+
+
+def test_transformer_group():
+    trans_group = TransformerGroup(7789, 8401)
+    assert len(trans_group.transformers) == 2
+    assert trans_group.transformers[0].name == "helmert"
+    assert trans_group.transformers[1].description == ("ITRF2014 to ETRF2014 (2)")
+    assert not trans_group.unavailable_operations
+    assert trans_group.best_available
+
+
+def test_transformer_group__unavailable():
+    trans_group = TransformerGroup(4326, 2964)
+    assert len(trans_group.unavailable_operations) == 1
+    assert (
+        trans_group.unavailable_operations[0].name
+        == "Inverse of NAD27 to WGS 84 (33) + Alaska Albers"
+    )
+    assert len(trans_group.transformers) == 8
+    assert trans_group.best_available
+
+
+def test_transform_group__missing_best():
+    with pytest.warns(DeprecationWarning):
+        lat_lon_proj = pyproj.Proj(init="epsg:4326", preserve_units=False)
+        alaska_aea_proj = pyproj.Proj(init="epsg:2964", preserve_units=False)
+
+    with pytest.warns(
+        UserWarning, match="Best transformation is not available due to missing Grid"
+    ):
+        trans_group = pyproj.transformer.TransformerGroup(
+            lat_lon_proj.crs, alaska_aea_proj.crs
+        )
+
+    assert not trans_group.best_available
+    assert len(trans_group.transformers) == 37
+    assert len(trans_group.unavailable_operations) == 41
+
+
+def test_transform_group__area_of_interest():
+    with pytest.warns(
+        UserWarning, match="Best transformation is not available due to missing Grid"
+    ):
+        trans_group = TransformerGroup(
+            4326, 2964, area_of_interest=AreaOfInterest(-136.46, 49.0, -60.72, 83.17)
+        )
+    assert (
+        trans_group.transformers[0].description
+        == "Inverse of NAD27 to WGS 84 (13) + Alaska Albers"
+    )
+
+
+def test_transformer__area_of_interest():
+    transformer = Transformer.from_crs(
+        4326, 2964, area_of_interest=AreaOfInterest(-136.46, 49.0, -60.72, 83.17)
+    )
+    assert transformer.description == "Inverse of NAD27 to WGS 84 (13) + Alaska Albers"
+
+
+def test_transformer_proj__area_of_interest():
+    transformer = Transformer.from_proj(
+        4326, 2964, area_of_interest=AreaOfInterest(-136.46, 49.0, -60.72, 83.17)
+    )
+    assert transformer.description == "Inverse of NAD27 to WGS 84 (13) + Alaska Albers"
+
+
+def test_transformer__area_of_interest__invalid():
+    with pytest.raises(ProjError):
+        Transformer.from_crs(
+            4326, 2964, area_of_interest=(-136.46, 49.0, -60.72, 83.17)
+        )
+
+
+def test_transformer_group__area_of_interest__invalid():
+    with pytest.raises(ProjError):
+        TransformerGroup(4326, 2964, area_of_interest=(-136.46, 49.0, -60.72, 83.17))
